/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import { world } from '@minecraft/server'; 
import { DYNAMIC_PROPERTY_IDS } from './../data.js';

export const PALETTES = {
    0: [
    '835432', '5e7c16', 'b02e26', '1d1d21',
    '9d9d97', '169c9c', '8932b8', '3c44aa',
    'fed83d', '80c71f', 'f38baa', '474f52',
    'f0f0f0', 'f9801d', 'c74ebd', '3ab3da'
    ],
    1: [
    '333333', '222222', '111111', '000000',
    '777777', '666666', '555555', '444444',
    'bbbbbb', 'aaaaaa', '999999', '888888',
    'ffffff', 'eeeeee', 'dddddd', 'cccccc'
    ],
    2: [
    'fcac2d', 'ff8c00', '7d3136', '671b20',
    'ff8052', 'ff5c20', '498574', '226e58',
    '7c3f3c', '672624', '4d3322', '5c4029',
    'ee817b', 'c2655b', '656b78', '797d8f'
    ],
    3: [
    '941fc6', '7200a3', '573d85', '45257a',
    'd157fe', 'a70cea', '7f40ba', '702cb0',
    'dfbf66', 'cbaa4e', '2d2d2d', '202020',
    'ffe397', 'eed181', '474747', '3a3a3a'
    ],
    4: [
    '8a8997', '90814d', '778272', '4c6156',
    'bfb755', 'aca55b', '6a7039', '507a32',
    '8eb971', '91bd59', '83b593', '80b497',
    '86b783', '88bb66', '79c05a', '64c73f'
    ]
}

export let CUSTOM_PALETTES = {}
export function updateCustomPalettes() {
    CUSTOM_PALETTES = {};
    for (let i = 0; i < 10; i++) {
        const paletteStr = world.getDynamicProperty(DYNAMIC_PROPERTY_IDS.CUSTOM_PALETTES + i);
        if (!paletteStr) continue;
        const customPalette = paletteStr.split('|');
        CUSTOM_PALETTES[customPalette.shift()] = customPalette;
    }
}
updateCustomPalettes();

export function getHexValue(hex = 'ffffff') {
    if (hex.includes('#')) hex = hex.replace('#', '');
    if (!/^#?[0-9A-Fa-f]{6}$/.test(hex)) hex = '555555';
    return hex;
}

export function hexToRGB(hex = 'FFFFFF') { return { r: parseInt(hex.substring(0, 2), 16), g: parseInt(hex.substring(2, 4), 16), b: parseInt(hex.substring(4, 6), 16) } }
export function rgbToHex({ r, g, b }) { return ((r << 16) | (g << 8) | b).toString(16).padStart(6, '0'); }
export function rgbToHsb({ r, g, b }) {
  const max = Math.max(r, g, b), min = Math.min(r, g, b), d = max - min;
  let h = 0;
  if (d) {
    if (max === r) h = ((g - b) / d + (g < b ? 6 : 0)) * 42.5;
    else if (max === g) h = ((b - r) / d + 2) * 42.5;
    else h = ((r - g) / d + 4) * 42.5;
  }
  const s = max === 0 ? 0 : (d / max) * 255;
  return { h, s, b: max };
}

export function getHue({ r, g, b }) {
    const max = Math.max(r, g, b), min = Math.min(r, g, b), d = max - min;
    let h = d ? max === r ? (g - b) / d % 6 : max === g ? (b - r) / d + 2 : (r - g) / d + 4 : 0;
    return Math.round(((h * 60 + 360) % 360) / 360 * 255);
}

export function hsbToRGB(h, s, b) {
  s /= 255; b /= 255; h /= 42.5;
  let c = b * s, x = c * (1 - Math.abs(h % 2 - 1)), m = b - c;
  let [r, g, bl] = h < 1 ? [c, x, 0] : h < 2 ? [x, c, 0] : h < 3 ? [0, c, x] : h < 4 ? [0, x, c] : h < 5 ? [x, 0, c] : [c, 0, x];
  return { r: Math.round((r + m) * 255), g: Math.round((g + m) * 255), b: Math.round((bl + m) * 255) };
}