/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
export const SHAPES = {
  square: ({ min, max, hollow = false }) => {
    const pixels = [];
    for (let x = Math.min(min.x, max.x); x <= Math.max(min.x, max.x); x++) {
      for (let y = Math.min(min.y, max.y); y <= Math.max(min.y, max.y); y++) {
        if (!hollow || x === min.x || x === max.x || y === min.y || y === max.y) pixels.push({ x, y });
      }
    }
    return pixels;
  },
  line: ({ min, max }) => {
    const pixels = [];
    let x = min.x, y = min.y;
    const dx = Math.abs(max.x - min.x), dy = Math.abs(max.y - min.y);
    const sx = min.x < max.x ? 1 : -1, sy = min.y < max.y ? 1 : -1;
    let err = dx - dy;
    while (true) {
      pixels.push({ x, y });
      if (x === max.x && y === max.y) break;
      const e2 = 2 * err;
      if (e2 > -dy) { err -= dy; x += sx; }
      if (e2 < dx) { err += dx; y += sy; }
    }
    return pixels;
  },
  circle: ({ min, max, hollow = false }) => {
    const pixels = [];
    const minX = Math.min(min.x, max.x), minY = Math.min(min.y, max.y);
    const maxX = Math.max(min.x, max.x), maxY = Math.max(min.y, max.y);
    const centerX = (minX + maxX) / 2, centerY = (minY + maxY) / 2;
    const radiusX = (maxX - minX + 1) / 2, radiusY = (maxY - minY + 1) / 2;
    for (let x = minX; x <= maxX; x++) {
      for (let y = minY; y <= maxY; y++) {
        const dx = (x - centerX) / radiusX, dy = (y - centerY) / radiusY;
        const distanceSquared = dx * dx + dy * dy;
        if (distanceSquared <= 1) {
          if (!hollow) { pixels.push({ x, y }); }
          else {
            const isInnerPixel = ((x - 1 >= minX && ((x - 1 - centerX) / radiusX) ** 2 + (dy) ** 2 <= 1) && (x + 1 <= maxX && ((x + 1 - centerX) / radiusX) ** 2 + (dy) ** 2 <= 1) && (y - 1 >= minY && (dx ** 2 + ((y - 1 - centerY) / radiusY) ** 2 <= 1)) && (y + 1 <= maxY && (dx ** 2 + ((y + 1 - centerY) / radiusY) ** 2 <= 1)));
            if (!isInnerPixel) pixels.push({ x, y });
          }
        }
      }
    }
    return pixels;
  }
};

export const allShapes = ['line|false', 'square|true', 'square|false', 'circle|true', 'circle|false'];