/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import { world } from '@minecraft/server';
import { ADDON_CONFIG } from './config/addonConfig.js';

const CANVAS_DEFAULT_COLOR = 16774883;

// * Addon Items
const ITEMS = {
    BRUSH: `${ADDON_CONFIG.NAMESPACE}:brush.item`,
    LARGE_BRUSH: `${ADDON_CONFIG.NAMESPACE}:large_brush.item`,
    PALETTE: `${ADDON_CONFIG.NAMESPACE}:palette.item`,
    EYEDROPPER: `${ADDON_CONFIG.NAMESPACE}:eyedropper.item`,
    EXPOSURE_TOOL: `${ADDON_CONFIG.NAMESPACE}:exposure_tool.item`,
    PAINT_BUCKET: `${ADDON_CONFIG.NAMESPACE}:paint_bucket.item`,
    PAINT_REMOVER: `${ADDON_CONFIG.NAMESPACE}:paint_remover.item`,
    SHAPES: `${ADDON_CONFIG.NAMESPACE}:shapes.item`,
    UNDO: `${ADDON_CONFIG.NAMESPACE}:undo.item`,
    REDO: `${ADDON_CONFIG.NAMESPACE}:redo.item`,
    CANVAS: `${ADDON_CONFIG.NAMESPACE}:canvas.item`,
    PAINTING: `${ADDON_CONFIG.NAMESPACE}:painting.item`,
    GUIDE_BOOK: `${ADDON_CONFIG.NAMESPACE}:guide_book.item`
}

// Addon Entities
const ENTITIES = {
    PLAYER_CONTROL: `${ADDON_CONFIG.NAMESPACE}:player_control.entity`,
    PALETTE_UI: `${ADDON_CONFIG.NAMESPACE}:palette_ui.entity`,
    SHAPE_BOX: `${ADDON_CONFIG.NAMESPACE}:shape_box.entity`,
    CANVAS: `${ADDON_CONFIG.NAMESPACE}:canvas_16x16.entity`,
    FRAME_CANVAS: `${ADDON_CONFIG.NAMESPACE}:frame_canvas_16x16.entity`
}

// Addon Tags
const TAGS = {
    DYNAMIC_PAINTINGS: `${ADDON_CONFIG.NAMESPACE}:item`,
    // Items
    BRUSH_ITEM: `${ADDON_CONFIG.NAMESPACE}:brush_item`,
    BRUSH_SIZE: `${ADDON_CONFIG.NAMESPACE}:brush_size-`,
    PAINTING_ITEM: `${ADDON_CONFIG.NAMESPACE}:painting_item`,
    PAINT_BUCKET_ITEM: `${ADDON_CONFIG.NAMESPACE}:paint_bucket_item`,
    PAINT_REMOVER_ITEM: `${ADDON_CONFIG.NAMESPACE}:paint_remover_item`,
    CANVAS_ITEM: `${ADDON_CONFIG.NAMESPACE}:canvas_item`,
    EYEDROPPER_ITEM: `${ADDON_CONFIG.NAMESPACE}:eyedropper_item`,
    
    // Blocks
    EASEL_BLOCK: `${ADDON_CONFIG.NAMESPACE}:easel`,
    FRAME_BLOCK: `${ADDON_CONFIG.NAMESPACE}:frame`,
    HAS_CANVAS: `${ADDON_CONFIG.NAMESPACE}:has_canvas`,
    
    // Entities
    IS_SAVED: `${ADDON_CONFIG.NAMESPACE}:is_saved`,
    PAINTING_TITLE: `${ADDON_CONFIG.NAMESPACE}:painting_title`,
    AUTHOR: `${ADDON_CONFIG.NAMESPACE}:author`
};

// Addon Particles
const PARTICLES = {
    BRUSH: `${ADDON_CONFIG.NAMESPACE}:brush_particle`,
    PAINT_BUCKET: `${ADDON_CONFIG.NAMESPACE}:paint_bucket_particle`,
    SUCCESS: `${ADDON_CONFIG.NAMESPACE}:success.particle`,
    DELETE: `${ADDON_CONFIG.NAMESPACE}:delete.particle`
};

// Property IDs
const PROPERTY_IDS = {
    PAINT_MODE: `${ADDON_CONFIG.NAMESPACE}:paint_mode`,
    POS_X: `${ADDON_CONFIG.NAMESPACE}:pos_x`,
    POS_Y: `${ADDON_CONFIG.NAMESPACE}:pos_y`,
    CURRENT_COLOR: `${ADDON_CONFIG.NAMESPACE}:current_color`,
    NEW_COLOR: `${ADDON_CONFIG.NAMESPACE}:new_color`,
    HUE_COLOR: `${ADDON_CONFIG.NAMESPACE}:hue_color`,
    HUE: `${ADDON_CONFIG.NAMESPACE}:hue`,
    PALETTE_TYPE: `${ADDON_CONFIG.NAMESPACE}:palette_type`,
    COLOR_RGB: `${ADDON_CONFIG.NAMESPACE}:color_rgb`,
    BRUSH_SIZE: `${ADDON_CONFIG.NAMESPACE}:brush_size`,
    POSITION_X: `${ADDON_CONFIG.NAMESPACE}:position_x`,
    POSITION_Y: `${ADDON_CONFIG.NAMESPACE}:position_y`,
    FROM_X: `${ADDON_CONFIG.NAMESPACE}:from_x`,
    FROM_Y: `${ADDON_CONFIG.NAMESPACE}:from_y`,
    TO_X: `${ADDON_CONFIG.NAMESPACE}:to_x`,
    TO_Y: `${ADDON_CONFIG.NAMESPACE}:to_y`
}

// Dynamic Property IDs
const DYNAMIC_PROPERTY_IDS = {
    IS_FIRST_LOAD: `${ADDON_CONFIG.NAMESPACE}:isFirstLoad`, 
    COLORS: `${ADDON_CONFIG.NAMESPACE}:colors`,
    CUSTOM_PALETTES: `${ADDON_CONFIG.NAMESPACE}:custom_palettes`
};

// Events
const EVENTS = {
    LINE: `${ADDON_CONFIG.NAMESPACE}:line`,
    SQUARE: `${ADDON_CONFIG.NAMESPACE}:square`
}

// ScriptEvent IDs
const SCRIPTEVENT_IDS = {
    DROP_ITEM: `${ADDON_CONFIG.NAMESPACE}:drop_item`,
    RESET_CUSTOM_PALETTES: `${ADDON_CONFIG.NAMESPACE}:reset_custom_palettes`
}

// Cardinal Rotations
const CARDINAL_ROTATIONS = { 'north': 0, 'east': 90, 'south': 180, 'west': 270 }

// Pixel Data
const PIXELS = new Map();
function _addPixels() {
    for (let x = 0; x < 16; x++) {
        for (let y = 0; y < 16; y++) {
            const group = `${ADDON_CONFIG.NAMESPACE}:px_${Math.floor(y / 4)}_${Math.floor(x / 4)}`;
            const index = `${ADDON_CONFIG.NAMESPACE}:px_${y % 4}_${x % 4}`;
            PIXELS.set(x + '|' + y, { group: group, index: index });
        }
    }
}
_addPixels();
function packRGB(r, g, b) { return (r << 16) | (g << 8) | b; }
function unPackRGB(rgb) { return { r: (rgb >> 16) & 0xFF, g: (rgb >> 8) & 0xFF, b: rgb & 0xFF }; }

export {
    CANVAS_DEFAULT_COLOR,
    ITEMS, ENTITIES, TAGS, PARTICLES, EVENTS,
    PIXELS,
    PROPERTY_IDS, DYNAMIC_PROPERTY_IDS,
    SCRIPTEVENT_IDS,
    CARDINAL_ROTATIONS,
    packRGB, unPackRGB
};