/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import EaselManager from './../../../managers/EaselManager.js';
import PlayerManager from './../../../managers/Player/PlayerManager.js';
import { ADDON_CONFIG } from './../../../config/addonConfig.js';
import { OBJECTIVES } from './../../../scores.js';
import { ITEMS, TAGS, PROPERTY_IDS } from './../../../data.js';

export const easelComp = {
    onPlayerInteract(eventData) {
        const { block, player } = eventData;
        if (player.isSneaking) return;
        if (OBJECTIVES.ACTIONS?.getScore(player) ?? 0 <= 0) return;
        const mainhand = PlayerManager.getItem(player, 'mainhand');
        const canvas = EaselManager.getCanvas(block);
        if (canvas) {
            if (canvas.getProperty(PROPERTY_IDS.PAINT_MODE)) return;
            if (mainhand?.hasTag(TAGS.BRUSH_ITEM) || mainhand?.hasTag(TAGS.PAINT_BUCKET_ITEM) || mainhand?.typeId === ITEMS.EYEDROPPER) EaselManager.startPaintMode(player, block, canvas, mainhand);
            else EaselManager.openForm(player, block, canvas);
        } else {
            if (mainhand?.hasTag(TAGS.CANVAS_ITEM)) EaselManager.addCanvas(block, player);
            else if (mainhand?.hasTag(TAGS.PAINTING_ITEM)) EaselManager.addPainting(block, player);
        }
    }
}

export const easelTickComp = {
    onTick({ block }) {
        const players = block.dimension.getEntities({
            type: 'minecraft:player',
            maxDistance: 2,
            location: block.bottomCenter()
        });
        if (players.filter(player => OBJECTIVES.ACTIONS?.getScore(player) ?? 0 <= 0).length === 0) {
            block.setPermutation(block.permutation.withState(`${ADDON_CONFIG.NAMESPACE}:paint_mode`, false));
        }
    }
}