/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import { ItemStack, GameMode } from '@minecraft/server';
import { ADDON_CONFIG } from './../../../config/addonConfig.js';
import EaselManager from './../../../managers/EaselManager.js';
import PlayerManager from './../../../managers/Player/PlayerManager.js';
import { ITEMS } from './../../../data.js';

const storedCanvasState = `${ADDON_CONFIG.NAMESPACE}:stored_canvas`;
const canvasItem = new ItemStack(ITEMS.CANVAS);
export const paintingCopierComp = {
    beforeOnPlayerPlace(eventData) {
        const { player, permutationToPlace: blockPerm } = eventData;
        if (player.getGameMode() === GameMode.creative) eventData.permutationToPlace = blockPerm.withState(storedCanvasState, 8);
    },
    onPlayerInteract(eventData) {
        const { block, player } = eventData;
        if (player.isSneaking) return;
        const mainhand = PlayerManager.getItem(player, 'mainhand');
        const storedCanvas = block.permutation.getState(storedCanvasState);
        if (mainhand?.typeId === ITEMS.CANVAS) {
            if (storedCanvas < 8) {
                block.setPermutation(block.permutation.withState(storedCanvasState, storedCanvas + 1));
                PlayerManager.decrementStack(player);
                player.playSound('use.wood');
            }
        } else if (mainhand?.typeId === ITEMS.PAINTING) {
            if (storedCanvas > 0) {
                block.setPermutation(block.permutation.withState(storedCanvasState, storedCanvas - 1));
                block.dimension.spawnItem(mainhand, block.above().bottomCenter());
                player.playSound('random.pop');
            }
        } else {
            if (storedCanvas > 0) {
                block.setPermutation(block.permutation.withState(storedCanvasState, storedCanvas - 1));
                block.dimension.spawnItem(canvasItem, block.above().bottomCenter());
                player.playSound('random.pop');
            }
        }
    },
    onPlayerDestroy(eventData) {
        const { block, destroyedBlockPermutation, player } = eventData;
        const storedCanvas = destroyedBlockPermutation.getState(storedCanvasState);
        if (storedCanvas === 0 || player.getGameMode() === GameMode.creative) return;
        for (let i = 0; i < storedCanvas; i++) {
            block.dimension.spawnItem(canvasItem, block.center());
        }
    }
}