/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import { world } from '@minecraft/server';
import { ADDON_CONFIG } from './../../config/addonConfig.js';
import { PAINT_MODE_CONFIG } from './../../config/paintModeConfig.js';
import { openGuideBookForm } from './../../guideBook.js';
import { OBJECTIVES, getPaintingAction, setActionScore } from './../../scores.js';
import { ITEMS, TAGS } from './../../data.js';

world.afterEvents.itemUse.subscribe((eventData) => {
    const { itemStack, source } = eventData;
    if (!itemStack?.hasTag(TAGS.DYNAMIC_PAINTINGS)) return;
    if (itemStack.typeId === ITEMS.GUIDE_BOOK) openGuideBookForm(source, 0);
    const scoreAction = OBJECTIVES.ACTIONS?.getScore(source) ?? 0;
    if (scoreAction <= 0) return;
    switch (itemStack.typeId) {
        case ITEMS.EYEDROPPER:
        if (PAINT_MODE_CONFIG.EYEDROPPER_SOUND) source.playSound('bottle.fill');
        setActionScore(source, 'eyedropper'); break;
        case ITEMS.PAINT_BUCKET:
        setActionScore(source, 'paint_bucket');
        source.startItemCooldown(`${ADDON_CONFIG.NAMESPACE}:bucket_item`, 20); break;
        case ITEMS.UNDO:
        setActionScore(source, 'undo');
        source.startItemCooldown(`${ADDON_CONFIG.NAMESPACE}:undo_redo_item`, 20); break;
        case ITEMS.REDO:
        setActionScore(source, 'redo');
        source.startItemCooldown(`${ADDON_CONFIG.NAMESPACE}:undo_redo_item`, 20); break;
        case ITEMS.SHAPES: {
            if (scoreAction === 100) setActionScore(source, 'shape_step_1');
            else if (scoreAction === 101) setActionScore(source, 'shape_step_2');
            source.startItemCooldown(`${ADDON_CONFIG.NAMESPACE}:shapes_item`, 20);
            break;
        }
    }
});

world.afterEvents.itemStartUse.subscribe((eventData) => {
    const { itemStack, source } = eventData;
    if (!itemStack?.hasTag(TAGS.DYNAMIC_PAINTINGS)) return;
    const scoreAction = OBJECTIVES.ACTIONS?.getScore(source) ?? 0;
    if (scoreAction <= 0) return;
    switch (itemStack.typeId) {
        case ITEMS.BRUSH: setActionScore(source, 'brush'); break;
        case ITEMS.LARGE_BRUSH: setActionScore(source, 'large_brush'); break;
        case ITEMS.PAINT_REMOVER: setActionScore(source, 'paint_remover'); break;
        case ITEMS.PALETTE: setActionScore(source, 'palette'); break;
        case ITEMS.EXPOSURE_TOOL: setActionScore(source, 'exposure_tool'); break;
    }
});

world.afterEvents.itemStopUse.subscribe((eventData) => {
    const { itemStack, source } = eventData;
    if (!itemStack?.hasTag(TAGS.DYNAMIC_PAINTINGS)) return;
    const scoreAction = OBJECTIVES.ACTIONS?.getScore(source) ?? 0;
    if (scoreAction <= 0) return;
    if (itemStack.typeId === ITEMS.BRUSH
    || itemStack.typeId === ITEMS.LARGE_BRUSH
    || itemStack.typeId === ITEMS.PAINT_REMOVER
    ) setActionScore(source, 'update_steps');
    else if (itemStack.typeId === ITEMS.EXPOSURE_TOOL) setActionScore(source, 'start_paint_mode');
    else if (itemStack.typeId === ITEMS.PALETTE) setActionScore(source, 'start_paint_mode');
});