/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import { ActionFormData, MessageFormData } from '@minecraft/server-ui';
import { ADDON_CONFIG } from './../config/addonConfig.js';
import { TAGS } from './../data.js'

export function EaselForm(canvas) {
    const tags = canvas.getTags();
    const paintingTitle = tags.find(tag => tag.startsWith(TAGS.PAINTING_TITLE))?.split('|')[1] ?? '---';
    const author = tags.find(tag => tag.startsWith(TAGS.AUTHOR))?.split('|')[1] ?? '---';
    const isSaved = canvas?.hasTag(TAGS.IS_SAVED) ?? false;
    const isSavedString = isSaved ? 'is_saved' : 'is_not_saved';
    const form = new ActionFormData().title('Custom Form' + `${ADDON_CONFIG.NAMESPACE}:easel_ui` + isSavedString);
    form.body({
        'rawtext': [
        { 'text': '\n' + paintingTitle + '§r§f\n'},
        { 'translate': 'ui.dynamic_paintings_gst.save_painting_ui.author' },
        { 'text': '\n' + author + '§r§f\n' }
        ]
    });
    if (!isSaved) form.button({ translate: 'ui.dynamic_paintings_gst.easel_ui.button_0_a' });
    if (isSaved) form.button({ translate: 'ui.dynamic_paintings_gst.easel_ui.button_0_b' });
    form.button('delete_painting');
    return form;
}

export function DeletePaintingForm() {
    const messageForm = new MessageFormData().title({ translate: 'ui.dynamic_paintings_gst.easel_ui.button_1' })
    .body({ translate: 'ui.dynamic_paintings_gst.delete_painting.message'})
    .button2({ translate: 'gui.cancel' })
    .button1({ translate: 'ui.dynamic_paintings_gst.confirm_button' });
    return messageForm;
}

export function FrameForm(canvas) {
    const tags = canvas.getTags();
    const paintingTitle = tags.find(tag => tag.startsWith(TAGS.PAINTING_TITLE))?.split('|')[1] ?? '---';
    const author = tags.find(tag => tag.startsWith(TAGS.AUTHOR))?.split('|')[1] ?? '---';
    const form = new ActionFormData();
    form.title('Custom Form' + `${ADDON_CONFIG.NAMESPACE}:easel_ui` + 'is_saved')
    .body({
        'rawtext': [
        { 'text': '\n' + paintingTitle + '§r§f\n'},
        { 'translate': 'ui.dynamic_paintings_gst.save_painting_ui.author' },
        { 'text': '\n' + author + '§r§f\n' }
        ]
    })
    .button({ translate: 'ui.dynamic_paintings_gst.easel_ui.button_0_b' });
    return form;
}