/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import { world } from '@minecraft/server'; 
import { ActionFormData, ModalFormData, MessageFormData } from '@minecraft/server-ui';
import { PALETTES, CUSTOM_PALETTES, updateCustomPalettes, getHexValue } from './../config/colorsConfig.js';
import { DYNAMIC_PROPERTY_IDS } from './../data.js';

export function PickColorForm() {
    const modalForm = new ModalFormData().title({ translate: 'ui.dynamic_paintings_gst.pick_color_form.title' })
    .textField({
        rawtext: [
        { translate: 'ui.dynamic_paintings_gst.pick_color_form.textfield_0' },
        { text: '\n\n' },
        { translate: 'ui.dynamic_paintings_gst.pick_color_form.textfield_1' },
        { text: '\n\n' },
        { translate: 'ui.dynamic_paintings_gst.pick_color_form.textfield_2' },
        { text: '\n\n' },
        { translate: 'ui.dynamic_paintings_gst.pick_color_form.textfield_3' },
        { text: '\n\n' }
        ]
    }, '#ffffff')
    .submitButton({ translate: 'ui.dynamic_paintings_gst.pick_color_form.select_color' });
    return modalForm;
}

export function PaletteForm() {
    const form = new ActionFormData().title({ translate: 'ui.dynamic_paintings_gst.palette_form.title' })
    .body({ 
        rawtext: [
        { text: '§e' },
        { translate: 'ui.dynamic_paintings_gst.palette_form.custom_palettes' },
        { text: `:§r§f ${Object.keys(CUSTOM_PALETTES).length}§e/10§r§f\n\n` },
        { translate: 'ui.dynamic_paintings_gst.palette_form.body_0' },
        { text: '\n\n' }
        ]
    });
    Object.keys(PALETTES).forEach((_, i) => {
        form.button({ translate: `ui.dynamic_paintings_gst.palette_form.palette_${i}` }, `textures/DynamicPaintings/ui/palettes/${i}`);
    });
    Object.keys(CUSTOM_PALETTES).forEach((palette) => {
        form.button(palette, 'textures/DynamicPaintings/ui/palettes/custom');
    });
    form.button({ translate: 'ui.dynamic_paintings_gst.palette_form.manage_palettes' }, 'textures/ui/icon_setting');
    return form;
}

export function ManageCustomPalettes() {
    const form = new ActionFormData().title({ translate: 'ui.dynamic_paintings_gst.palette_form.manage_palettes' })
    .body({
        rawtext: [
        { text: '§e' },
        { translate: 'ui.dynamic_paintings_gst.palette_form.custom_palettes' },
        { text: `:§r§f ${Object.keys(CUSTOM_PALETTES).length}§e/10§r§f\n\n` },
        { translate: 'ui.dynamic_paintings_gst.palette_form.manage_palettes.tip' },
        { text: '\n\n§f/function reset_custom_palettes§r\n\n' }
        ]
    });
    const customPalettes = Object.keys(CUSTOM_PALETTES);
    customPalettes.forEach((palette) => {
        form.button(palette, 'textures/DynamicPaintings/ui/palettes/custom');
    });
    if (customPalettes.length < 10) { form.button({ translate: 'ui.dynamic_paintings_gst.palette_form.add_palette' }, 'textures/DynamicPaintings/ui/add'); }
    return form;
}

export function AddCustomPalettes(player) {
    const modalForm = new ModalFormData().title({ translate: 'ui.dynamic_paintings_gst.palette_form.add_palette' });
    modalForm.textField({ translate: 'ui.dynamic_paintings_gst.palette_form.add_palette.palette_name' }, '...');
    for (let i = 0; i < 16; i++) {
        modalForm.textField({ translate: 'ui.dynamic_paintings_gst.palette_form.add_palette.color', with: [`${i + 1}`] }, '#FFFFFF');
    }
    modalForm.show(player).then((formData) => {
        if (formData.canceled) return;
        const formValues = formData.formValues;
        let paletteName = formValues.shift();
        if (Object.keys(CUSTOM_PALETTES).includes(paletteName)) return;
        let paletteStr = paletteName + '|';
        for (let i = formValues.length - 4; i >= 0; i -= 4) {
            for (let j = 0; j < 4; j++) {
                paletteStr += `${getHexValue(formValues[i + j])}|`;
            }
        }
        if (paletteStr.endsWith('|')) paletteStr = paletteStr.slice(0, -1);
        world.setDynamicProperty(DYNAMIC_PROPERTY_IDS.CUSTOM_PALETTES + Object.keys(CUSTOM_PALETTES).length, paletteStr);
        updateCustomPalettes();
    });
}

export function DeleteCustomPalettes(player, paletteIndex = 0) {
    new MessageFormData().title({ translate: 'ui.dynamic_paintings_gst.palette_form.delete_palette' })
    .body({ translate: 'ui.dynamic_paintings_gst.palette_form.delete_palette.message' })
    .button2({ translate: 'gui.cancel' })
    .button1({ translate: 'ui.dynamic_paintings_gst.confirm_button' })
    .show(player).then((formData) => {
        if (formData.canceled || formData.selection === 1) return;
        world.setDynamicProperty(DYNAMIC_PROPERTY_IDS.CUSTOM_PALETTES + paletteIndex, '');
        for (let i = paletteIndex; i < Object.keys(CUSTOM_PALETTES).length - 1; i++) { world.setDynamicProperty(DYNAMIC_PROPERTY_IDS.CUSTOM_PALETTES + i, world.getDynamicProperty(DYNAMIC_PROPERTY_IDS.CUSTOM_PALETTES + (i + 1))); }
        world.setDynamicProperty(DYNAMIC_PROPERTY_IDS.CUSTOM_PALETTES + (Object.keys(CUSTOM_PALETTES).length - 1), '');
        updateCustomPalettes();
    });
}