/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import { MolangVariableMap, system } from '@minecraft/server';
import PlayerManager from './../../Player/PlayerManager.js';
import { setActionScore } from './../../../scores.js';
import { PARTICLES, PIXELS, CANVAS_DEFAULT_COLOR, packRGB, unPackRGB } from './../../../data.js';
// --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function getPixel(point, canvasPixels) {
    if (!point) return; 
    const pixel = PIXELS.get(point.x + '|' + point.y);
    if (!pixel) return;
    const group = canvasPixels.find(px => px.hasTag(pixel.group));
    if (!group) return;
    return { group: group, index: pixel.index };
}
function getPixelColor(player, point, canvasPixels, currentColor, updateScore = true) {
    if (updateScore) setActionScore(player, 'start_paint_mode');
    if (!point) return currentColor;
    const pixel = getPixel(point, canvasPixels);
    if (!pixel) return currentColor;
    return pixel.group.getProperty(pixel.index);
}
// --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
function clearPixel(point, canvasPixels) {
    if (!point) return;
    const pixel = getPixel(point, canvasPixels);
    pixel.group.setProperty(pixel.index, CANVAS_DEFAULT_COLOR);
}
function paintPixel(player, point, pixels, colorToPaint = 0, brushSize = 1, allowSound = true, allowParticles = true) {
    if (!point) return;
    const pixel = getPixel(point, pixels);
    if (!pixel) { setActionScore(player, 'update_steps'); return; }
    if (brushSize === 3) {
        const {x, y} = point, nearbyPoints = [{ x: x + 1, y: y },{ x: x - 1, y: y },{ x: x, y: y + 1 },{ x: x, y: y - 1 },{ x: x + 1, y: y + 1 },{ x: x - 1, y: y - 1 },{ x: x + 1, y: y - 1 },{ x: x - 1, y: y + 1 }];
        paintPixel(player, point, pixels, colorToPaint, 1, allowSound, allowParticles);
        for (const nearbyPoint of nearbyPoints) { if (nearbyPoint.x >= 0 && nearbyPoint.x <= 15 && nearbyPoint.y >= 0 && nearbyPoint.y <= 15) paintPixel(player, nearbyPoint, pixels, colorToPaint, 1, false, false); }
        return;
    }
    if (allowSound && Math.random() > 0.80) player.playSound('brush.generic');
    if (allowParticles && Math.random() > 0.75) {
        const rgb = unPackRGB(colorToPaint);
        const mol = new MolangVariableMap(); mol.setColorRGB('color', { red: rgb.r/255, green: rgb.g/255, blue: rgb.b/255 });
        player.dimension.spawnParticle(PARTICLES.BRUSH, PlayerManager.getViewPoint(player, 0.75, 1.5), mol);
    }
    pixel.group.setProperty(pixel.index, colorToPaint);
}
function fillPixels(player, point, canvasPixels, colorToPaint = 0, allowSound = true, allowParticles = true) {
    if (allowSound) player.playSound('bucket.empty_water');
    if (allowParticles) {
        const rgb = unPackRGB(colorToPaint);
        const mol = new MolangVariableMap(); mol.setColorRGB('color', { red: rgb.r/255, green: rgb.g/255, blue: rgb.b/255 });
        player.dimension.spawnParticle(PARTICLES.PAINT_BUCKET, PlayerManager.getViewPoint(player, 0.75, 1.5), mol);
    }
    const previousColor = getPixelColor(player, point, canvasPixels, colorToPaint);
    if (previousColor === colorToPaint) { setActionScore(player, 'start_paint_mode'); return; }
    const stack = [point], paintedPixels = new Set();
    paintedPixels.add(point.x + '|' + point.y);
    while (stack.length > 0) {
        const { x, y } = stack.pop(), currentPoint = { x, y };
        if (getPixelColor(player, currentPoint, canvasPixels, colorToPaint) === previousColor) {
            paintPixel(player, currentPoint, canvasPixels, colorToPaint, 1, false, false);
            const nearbyPoints = [{ x: x - 1, y },{ x: x + 1, y },{ x, y: y - 1 },{ x, y: y + 1 }];
            for (const nearbyPoint of nearbyPoints) {
                const key = nearbyPoint.x + '|' + nearbyPoint.y;
                if (nearbyPoint.x >= 0 && nearbyPoint.x <= 15 && nearbyPoint.y >= 0 && nearbyPoint.y <= 15 && !paintedPixels.has(key)) {
                    stack.push(nearbyPoint);
                    paintedPixels.add(key);
                }
            }
        }
    }
    setActionScore(player, 'update_steps');
}

function changePixelBrightness(color = 0, value = 0) {
    const RGB = unPackRGB(color);
    return packRGB(Math.min(255, Math.max(0, RGB.r + value)), Math.min(255, Math.max(0, RGB.g + value)), Math.min(255, Math.max(0, RGB.b + value)));
}

export {
    getPixel, getPixelColor,
    clearPixel, paintPixel, fillPixels,
    changePixelBrightness
}