/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
const _centerYaw = { 'north': 0, 'east': 90, 'south': 180, 'west': 270 };
function getViewPoint(player, cardinalDirection = 'north', horizontalFOV = 60, verticalFOV = 60, yOffset = -5) {
    const normY = ((player.getRotation().x + (verticalFOV / 2)) / verticalFOV) * 16;
    return { x: getViewPointHorizontal(player, cardinalDirection, horizontalFOV, 16), y: 15 - Math.max(0, Math.min(15, Math.floor(normY) + yOffset)) }
}

function getViewPointHorizontal(player, cardinalDirection, horizontalFOV = 90, max = 16) {
    let centerYaw = _centerYaw[cardinalDirection],
    playerYaw = player.getRotation().y % 360;
    if (playerYaw < 0) playerYaw += 360;
    let yawOffset = playerYaw - centerYaw;
    if (yawOffset < -180) yawOffset += 360;
    if (yawOffset > 180) yawOffset -= 360;
    yawOffset = Math.max(-horizontalFOV / 2, Math.min(horizontalFOV / 2, yawOffset));
    const normX = ((yawOffset + (horizontalFOV / 2)) / horizontalFOV) * max ;
    return Math.max(0, Math.min(max - 1, Math.floor(normX)));
}

function getViewPointForPalette(player, cardinalDirection = 'north', horizontalFOV = 40, verticalFOV = 60, yOffset = -5, max = { x: 256, y: 256 }) {
    const normY = ((player.getRotation().x + (verticalFOV / 2)) / verticalFOV) * max.y;
    return { x: getViewPointHorizontal(player, cardinalDirection, horizontalFOV, max.x), y: max.y-1 - Math.max(0, Math.min(max.y -1, Math.floor(normY) + yOffset)) }
}

export { getViewPoint, getViewPointHorizontal, getViewPointForPalette }