/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import { ADDON_CONFIG } from './../config/addonConfig.js';
import PaintModeManager from './PaintModeManager.js';
import { SavePaintingForm } from './../forms/SavePaintingForm.js';
import { TAGS, DYNAMIC_PROPERTY_IDS, PIXELS, PARTICLES } from './../data.js';

function getPointFromIndex(index) {
    return { x: 15 - (index % 16), y: 15 - (Math.floor(index / 16)) };
}

class PaintingManager {
    static savePainting(player, canvas) {
        SavePaintingForm(player).show(player).then((e) => {
            if (e.canceled) return;
            let paintingTitle = e.formValues[0];
            let author = e.formValues[1];
            if (paintingTitle.length > 26) paintingTitle = paintingTitle.slice(0, 23) + '...';
            if (author.length > 30) author = author.slice(0, 27) + '...';
            canvas.addTag(TAGS.IS_SAVED);
            canvas.addTag(TAGS.PAINTING_TITLE + '|' + paintingTitle);
            canvas.addTag(TAGS.AUTHOR + '|' + author);
            canvas.dimension.spawnParticle(PARTICLES.SUCCESS, { x: canvas.location.x, y: canvas.location.y +0.5, z: canvas.location.z });
        });
    }
    
    static getPaintingColors(canvas) {
        let colors = '';
        const pixelGroups = canvas.dimension.getEntities({ location: canvas.location, families: [`${ADDON_CONFIG.NAMESPACE}.pixels`], closest: 16 });
        for (let x = 0; x < 16; x++) {
            for (let y = 0; y < 16; y++) {
                const rotX = 15 - y;
                const rotY = 15 - x;
                const pixel = PIXELS.get(rotX + '|' + rotY);
                const group = pixelGroups.find(px => px.hasTag(pixel.group));
                if (group) {
                    const colorRGB = group.getProperty(pixel.index);
                    colors += (x === 15 && y === 15) ? colorRGB : `${colorRGB}|`;
                }
            }
        }
        return colors;
    }
    
    static loadPaintingColors(canvasItem, canvas) {
        const colors = canvasItem.getDynamicProperty(DYNAMIC_PROPERTY_IDS.COLORS)?.split('|');
        const pixelGroups = canvas.dimension.getEntities({ location: canvas.location, families: [`${ADDON_CONFIG.NAMESPACE}.pixels`], closest: 16 });
        colors.forEach((colorRGB, index) => {
            const { x, y } = getPointFromIndex(index);
            const pixel = PIXELS.get(x + '|' + y);
            const group = pixelGroups.find(px => px.hasTag(pixel.group));
            if (group) {
                const currentColor = group.getProperty(pixel.index);
                if (currentColor != colorRGB) group.setProperty(pixel.index, parseInt(colorRGB));
            }
        });
    }
    
    static loadStep(canvas, step) {
        if (!canvas || !step) return;
        const colors = step.split('|');
        const pixelGroups = canvas.dimension.getEntities({ location: canvas.location, families: [`${ADDON_CONFIG.NAMESPACE}.pixels`], closest: 16 });
        colors.forEach((colorRGB, index) => {
            const { x, y } = getPointFromIndex(index);
            const pixel = PIXELS.get(x + '|' + y);
            const group = pixelGroups.find(px => px.hasTag(pixel.group));
            if (group) {
                const currentColor = group.getProperty(pixel.index);
                if (currentColor != colorRGB) group.setProperty(pixel.index, parseInt(colorRGB));
            }
        });
    }
}

export default PaintingManager;