/**
* Copyright © [2024] [GST378]. All rights reserved.
* Personal use only. Modifications and redistribution are prohibited.
* See LICENSE file for details.
*/
import {
    EntityEquippableComponent,
    EquipmentSlot,
    EntityInventoryComponent,
    ItemDurabilityComponent,
    ItemEnchantableComponent,
    GameMode
} from '@minecraft/server';

class PlayerManager {
    
    static hasItem(player, item, slot = 'mainhand', index = 0) {
        return PlayerManager.getItem(player, slot, index) === item;
    }
    
    static getItem(player, slot, index = 0) {
        const equippableComp = player.getComponent(EntityEquippableComponent.componentId);
        switch (slot) {
            case 'inventory': return  player.getComponent(EntityInventoryComponent.componentId).container.getItem(index);
            case 'mainhand': return equippableComp.getEquipment(EquipmentSlot.Mainhand);
            case 'offhand': return equippableComp.getEquipment(EquipmentSlot.Offhand);
            case 'head': return equippableComp.getEquipment(EquipmentSlot.Head);
            case 'chest': return equippableComp.getEquipment(EquipmentSlot.Chest);
            case 'legs': return equippableComp.getEquipment(EquipmentSlot.Legs);
            case 'feet': return equippableComp.getEquipment(EquipmentSlot.Feet);
            default: return undefined;
        }
    }
    
    static decrementStack(player, quantity = 1, allGameModes = false) {
        if (!player.isValid()) return;
        if (!allGameModes && player.getGameMode() === GameMode.creative) return;
        const mainhand = PlayerManager.getItem(player, 'mainhand');
        if (!mainhand) return;
        const inventory = player.getComponent(EntityInventoryComponent.componentId).container;
        let amount = mainhand.amount;
        if (amount <= quantity) inventory.setItem(player.selectedSlotIndex, null);
        else {
            mainhand.amount -= quantity;
            inventory.setItem(player.selectedSlotIndex, mainhand);
        }
    }
    
    static decreaseDurability(player, item, allGameModes = false) {
        if (!player.isValid() || !item) return;
        if (!allGameModes && player.getGameMode() === GameMode.creative) return;
        const inventory = player.getComponent(EntityInventoryComponent.componentId).container;
        const durability = item.getComponent(ItemDurabilityComponent.componentId);
        if (!durability) return;
        const unbreakingLevel = item.getComponent(ItemEnchantableComponent.componentId)?.getEnchantment('unbreaking')?.level;
        if (Math.random() > durability.getDamageChance(unbreakingLevel) / 100) return;
        if (durability.damage === durability.maxDurability) {
            inventory.setItem(player.selectedSlotIndex, undefined);
            player.playSound('random.break');
        } else {
            durability.damage++;
            inventory.setItem(player.selectedSlotIndex, item);
        }
    }
    
    static getCardinalDirection(player) {
        const yaw = player.getRotation().y;
        if (yaw >= -45 && yaw < 45) return 'north';
        else if (yaw >= 45 && yaw < 135) return 'east';
        else if (yaw >= 135 || yaw < -135) return 'south';
        else return 'west';
    }
    
    static getViewPoint(player, distance = 1, height = 1) {
        const viewDir = player.getViewDirection();
        const { x, y, z } = player.location;
        return {
            x: x + (viewDir.x * distance),
            y: (y + height) + viewDir.y,
            z: z + (viewDir.z * distance)
        }
    }
}

export default PlayerManager;